<?php
namespace EMS\PortalFuncionario\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\{ Message, Error };

use EMS\PortalFuncionario\Geral\Services\OcorrenciaService;

class OcorrenciaController extends Crud {
    /** @var OcorrenciaService */
    protected $ocorrenciaService;

    public function __construct(
        EntityManager $entityManager,
        OcorrenciaService $ocorrenciaService
    ) {
        $this->entityManager        = $entityManager;
        $this->ocorrenciaService    = $ocorrenciaService;
    }

    public function getOcorrenciasPendentes(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $params = $request->getParameters();

            $nrorg = $params['NRORG'];
            $nrvinculom = $params['NRVINCULOM'];
            $nrperiodoapuracao = empty($params['NRPERIODOAPURACAO']) ? null : $params['NRPERIODOAPURACAO'];

            $totalOcorrenciasPendentes = $this->ocorrenciaService->getOcorrenciasPendentes(
                $nrorg,
                $nrvinculom,
                $nrperiodoapuracao
            );
            $response->addDataSet(new DataSet('data', $totalOcorrenciasPendentes));
        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getOcorrenciasPeriodo(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $params = $request->getParameters();

            $nrorg = $params['NRORG'];
            $nrvinculom = $params['NRVINCULOM'];
            $nrperiodoapuracao = empty($params['NRPERIODOAPURACAO']) ? null : $params['NRPERIODOAPURACAO'];

            $totalOcorrenciasPendentes = $this->ocorrenciaService->getOcorrenciasPeriodo(
                $nrorg,
                $nrvinculom,
                $nrperiodoapuracao
            );
            $response->addDataSet(new DataSet('data', $totalOcorrenciasPendentes));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getMotivoOcorFreq(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $params = $request->getParameters();

            $nrorg = $params['NRORG'];

            $totalOcorrenciasPendentes = $this->ocorrenciaService->getMotivoOcorFreq(
                $nrorg
            );
            $response->addDataSet(new DataSet('data', $totalOcorrenciasPendentes));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getMotivoOcorFreqLancamentoRetroativo(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $params = $request->getParameters();

            $nrorg = $params['NRORG'];

            $totalOcorrenciasPendentes = $this->ocorrenciaService->getMotivoOcorFreqLancamentoRetroativo(
                $nrorg
            );
            $response->addDataSet(new DataSet('data', $totalOcorrenciasPendentes));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function setOcorrenciaFreq(Request\Row $request, DTO\Response $response){
        try {
            $row = $request->getRow();
    
            $success = $this->ocorrenciaService->setOcorrenciaFreq($row);

            $response->addDataSet(new DataSet('setOcorrenciaFreq', array('success' => $success)));
        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function setOcorrenciasFreq(
        Request\Row $request,
        Response $response
    ){
        try {
            $row = $request->getRow();

            $success = $this->ocorrenciaService->
                setOcorrenciasFreq($row);

            $dataset = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('setOcorrenciasFreq', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function removeOcorrenciaFreq(Request\Row $request, DTO\Response $response){
        try {
            $row = $request->getRow();
    
            $success = $this->ocorrenciaService->removeOcorrenciaFreq($row);

            $response->addDataSet(new DataSet('removeOcorrenciaFreq', array('success' => $success)));
        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function removeOcorrenciasFreq(Request\Row $request, DTO\Response $response){
        try {
            $row = $request->getRow();

            $success = $this->ocorrenciaService->
                removeOcorrenciasFreq($row);

            $dataset = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('removeOcorrenciasFreq', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function removeAlmocos(Request\Row $request, DTO\Response $response){
        try {
            $row = $request->getRow();

            $nrorg      =           $row['NRORG'];
            $nrvinculom =           $row['NRVINCULOM'];
            $date       = (array)   $row['DATE'];

            $success = $this->ocorrenciaService->removeAlmocos(
                $nrorg,
                $nrvinculom,
                $date
            );

            $dataset = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('removeAlmocos', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTpOcorrencia(DTO\Request\Filter $request, Response $response) {
        try {
            $params = $request->getParameters();
            $nrOrg             = $params['NRORG'];

            $dataSet = $this->ocorrenciaService->
                getTpOcorrencia($nrOrg);
            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function insertAlmocoMarcacaoRetroativa(
        Request\Row $request,
        Response $response
    ){
        try {
            $row = $request->getRow();

            $nrOrg      = (int)     $row['NRORG'];
            $cdOperador = (string)  $row['CDOPERADOR'];
            $nrVinculo  = (int)     $row['NRVINCULOM'];
            $dates      = (array)   $row['DATE'];

            $success = $this->ocorrenciaService->
                insertAlmocoMarcacaoRetroativa($nrOrg, $cdOperador, $nrVinculo, $dates);

            $dataset = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('data', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getMotivosOcorrencia(
        DTO\Request\Filter $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getParameters();
            $motivos = $this->ocorrenciaService->getMotivosOcorrencia($row);
            $response->addDataSet(new DataSet('data', $motivos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }

    public function insereOcorrenciasAlmoco(
        DTO\Request\Row     $request,
        DTO\Response        $response
    ) {

        try {
            $row = $request->getRow();

            $nrOrg          = (int)     $row['NRORG'];
            $nrVinculo      = (int)     $row['NRVINCULOM'];
            $dia            = (array)   $row['DATE'];
            $horaMarcacão   = (string)  $row['HORAMARCACAO'];
            $cdOperador     = (string)  $row['CDOPERADOR'];

            $success = $this->ocorrenciaService->insereOcorrenciasAlmoco(
                $nrOrg,
                $nrVinculo,
                $cdOperador,
                $horaMarcacão,
                $dia
            );

            $dataSet = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function solicitaCompensacaoFaltas(
        DTO\Request\Row     $request,
        DTO\Response        $response
    ) {

        try {
            $row = $request->getRow();

            $nrOrg          = (int)     $row['NRORG'];
            $nrVinculo      = (int)     $row['NRVINCULOM'];
            $dia            = (array)   $row['DATE'];

            $status = $this->ocorrenciaService->solicitaCompensacaoFaltas(
                $nrOrg,
                $nrVinculo,
                $dia
            );

            if ($status['success']) {
                $dataSet = [
                    'success' => true
                ];

                $response->addDataSet(new DataSet('data', $dataSet));
            } else {
                $message = $status['message'];
                $response->addMessage(new Message($message, Message::TYPE_ERROR));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function solicitaAbonoFaltas(
        DTO\Request\Row     $request,
        DTO\Response        $response
    ) {

        try {
            $row = $request->getRow();

            $nrOrg          = (int)     $row['NRORG'];
            $nrVinculo      = (int)     $row['NRVINCULOM'];
            $dia            = (array)   $row['DATE'];
            $dsobservacao   = (string)  $row['DSOBSERVACAO'];
            $file           = (string)  $row['FILE'];
            $nmfile         = (string)  ($row['NMFILE'] ?? null);
            $nrMotivo       = (string)  $row['NRMOTIVOOCORFREQ'];
            $cdTabecdi      = (string)  isset($row['CDTABECDI'])   ? $row['CDTABECDI']     : null;
            $cdDiagnost     = (string)  isset($row['CDDIAGNOST'])  ? $row['CDDIAGNOST']    : null;
            $nrPessoa       = (string)  isset($row['NRPESSOA'])    ? $row['NRPESSOA']      : null;

            $dataSet = $this->ocorrenciaService->solicitaAbonoFaltas(
                $nrOrg,
                $nrVinculo,
                $dia,
                $dsobservacao,
                $file,
                $nmfile,
                $nrMotivo,
                $cdTabecdi,
                $cdDiagnost,
                $nrPessoa
            );

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function solicitarCompensacaoHoras(
        Request\Row     $request,
        Response        $response
    ) {

        try {
            $row = $request->getRow();

            $nrOrg          = (int)     $row['NRORG'];
            $nrVinculo      = (int)     $row['NRVINCULOM'];
            $dia            = (string)  $row['DATE'];

            $status = $this->ocorrenciaService->solicitarCompensacaoHoras(
                $nrOrg, $nrVinculo, $dia
            );

            if ($status['success']) {
                $dataSet = [
                    'success' => true
                ];

                $response->addDataSet(new DataSet('data', $dataSet));
            } else {
                $message = $status['message'];
                $response->addMessage(new Message($message, Message::TYPE_ERROR));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function solicitarHorasExtras(
        Request\Row     $request,
        Response        $response
    ) {

        try {
            $row = $request->getRow();

            $nrOrg          = (int)     $row['NRORG'];
            $nrVinculo      = (int)     $row['NRVINCULOM'];
            $observacao     = (string)  $row['DSOBSERVACAO'];
            $file           = (string)   ($row['FILE'] ?? null);
            $dia            = (string)  $row['DATE'];
            $cdOperador     =           $row['CDOPERADOR'] ?? null;
            
            $status = $this->ocorrenciaService->solicitarHorasExtras(
                $nrOrg, $nrVinculo, $observacao, $file, $dia, $cdOperador
            );

            if ($status['success']) {
                $dataSet = [
                    'success' => true
                ];

                $response->addDataSet(new DataSet('data', $dataSet));
            } else {
                $message = $status['message'];
                $response->addMessage(new Message($message, Message::TYPE_ERROR));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getOcorrenciaFile(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg          = (int) $row['NRORG'];
            $nrOcorrencia   = (int) $row['NRPROGOCORRENCIA'];

            $file = $this->ocorrenciaService->getOcorrenciaFile(
                $nrOrg,
                $nrOcorrencia
            );

            $response->addDataSet(new DataSet('data', $file));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getDiagnost(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();
            $nrOrg = (int) $row['NRORG'];

            $pessoaConselhoReg = $this->ocorrenciaService->getDiagnost(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $pessoaConselhoReg));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getPessoaConselhoReg(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();
            $nrOrg = (int) $row['NRORG'];

            $pessoaConselhoReg = $this->ocorrenciaService->getPessoaConselhoReg(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $pessoaConselhoReg));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
}