<?php

namespace EMS\PortalFuncionario\Geral\Services;

use EMS\PortalFuncionario\Util\HcmServices;

class DocumentoService {
    /** @var HcmServices */
    protected $hcmServices;

    public function __construct(
        HcmServices $hcmServices
    ) {
        $this->hcmServices = $hcmServices;
    }

    public function getDocumentosVinculo(
        int $nrOrg,
        int $nrVinculo,
            $nrCategoria
    ): array {
        $route = 'getDocumentosVinculo';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM' => $nrVinculo,
                'NRCATEGDOCUMENTO' => $nrCategoria,
            ],
        ];

        $result = $this->hcmServices->send($route, $params, $nrOrg);
        $documentos = $result['dataset'][$route];

        return $documentos;
    }

    public function getCategoriasComDocumentoVinculo(
        int $nrOrg,
        int $nrVinculo
    ): array {
        $route = 'getCategoriasComDocumentoVinculo';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM' => $nrVinculo,
            ],
        ];

        $result = $this->hcmServices->send($route, $params, $nrOrg);
        $categorias = $result['dataset'][$route];

        return $categorias;
    }

    public function getCategorias(
        int $nrOrg
    ): array {
        $route = 'getCategorias';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'EMPTY' => '',
            ],
        ];

        $result = $this->hcmServices->send($route, $params, $nrOrg);
        $categorias = $result['dataset'][$route];

        return $categorias;
    }

    public function getTipos(
        int $nrOrg
    ): array {
        $route = 'getTipos';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'EMPTY' => '',
            ],
        ];

        $result = $this->hcmServices->send($route, $params, $nrOrg);
        $tipos = $result['dataset'][$route];

        return $tipos;
    }

    public function downloadDocumento(
        int $nrOrg,
        int $nrAnexo
    ): array {
        $route = 'downloadDocumento';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRANEXO' => $nrAnexo,
            ],
        ];

        $result = $this->hcmServices->send($route, $params, $nrOrg);
        $file = $result['dataset'][$route];

        return $file;
    }

    public function saveDocumento(
        array $row
    ): bool {
        $route = 'saveDocumento';

        $nrOrg = (int) $row['NRORG'];

        $params = [
            'requestType' => 'Row',
            'row' => $row,
        ];

        $result = $this->hcmServices->send($route, $params, $nrOrg);
        $success = $result['dataset'][$route]['success'];

        return $success;
    }

    public function getAssinaturasVinculo(
        int $nrOrg,
        int $nrVinculo,
        int $nrperiodoapuracao
    ): array {
        $route = 'getAssinaturasVinculo';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM' => $nrVinculo,
                'NRPERIODOAPURACAO' => $nrperiodoapuracao
            ],
        ];

        $result = $this->hcmServices->send($route, $params, $nrOrg);
        $documentos = $result['dataset'][$route];

        return $documentos;
    }

    public function getAssinaturaByVinculo(
        int $nrOrg,
        int $nrVinculo
    ): array {
        $route = 'getAssinaturaByVinculo';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM' => $nrVinculo,
            ],
        ];

        $result = $this->hcmServices->send($route, $params, $nrOrg);
        $assinatura = $result['dataset'][$route];

        return $assinatura;
    }

    public function saveDocumentoAssinatura(
        array $row
    ): bool {
        $route = 'saveDocumentoAssinatura';

        $nrOrg = (int) $row['NRORG'];

        $params = [
            'requestType' => 'Row',
            'row' => $row,
        ];

        $result = $this->hcmServices->send($route, $params, $nrOrg);
        $success = $result['dataset'][$route]['success'];

        return $success;
    }

    public function saveAssinaturaVinculo(
        array $row
    ): bool {
        $route = 'saveAssinaturaVinculo';

        $nrOrg = (int) $row['NRORG'];

        $params = [
            'requestType' => 'Row',
            'row' => $row,
        ];

        $result = $this->hcmServices->send($route, $params, $nrOrg);
        $success = $result['dataset'][$route]['success'];

        return $success;
    }

    public function getIntegracaoDigitalFile(
        int $nrOrg,
        int $nrVinculo,
        int $nrParcNegocio,
        int $nrIntegracaoDigital
    ): array {
        $route = 'getIntegracaoDigitalFile';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM' => $nrVinculo,
                'NRPARCNEGOCIO' => $nrParcNegocio,
                'NRINTELETRDIGIT' => $nrIntegracaoDigital
            ],
        ];

        $result = $this->hcmServices->send($route, $params, $nrOrg);
        $assinatura = $result['dataset'][$route];

        return $assinatura;
    }

    public function changeStatusAssinatura(
        array $row
    ): bool {
        $route = 'changeStatusAssinatura';

        $nrOrg = (int) $row['NRORG'];

        $params = [
            'requestType' => 'Row',
            'row' => $row,
        ];

        $result = $this->hcmServices->send($route, $params, $nrOrg);
        $success = $result['dataset'][$route]['success'];

        return $success;
    }
}