<?php
namespace EMS\PortalFuncionario\Geral\Services;

use EMS\PortalFuncionario\Util\HcmServices;
use EMS\PortalFuncionario\Geral\Services\ParamOrgService;

class ReportService {

    /** @var HcmServices */
    protected $hcmServices;

    public function __construct(
        HcmServices $hcmServices,
        ParamOrgService $paramOrgService
    ) {
        $this->hcmServices = $hcmServices;
        $this->paramOrgService                  = $paramOrgService;
    }

    public function generateBirtReport(
        array $row
    ): string {
        $nrOrg = (int) $row['NRORG'];
        $dtmescompetenc = $row['P_COMPETENCIA'] ?? null;
        $hcmServicesRoute = 'generateBirtReport';

        try {
            $anoReferencia = $this->paramOrgService->getValorReferenciaPF($nrOrg, 'ANO_INICIO', date('01/m/Y'));

            $anoCompetencia = date('Y', strtotime($dtmescompetenc));

            if ((int)$anoCompetencia < (int)$anoReferencia) {
                $inicioAno = '01/01/' . $anoReferencia;
                throw new \Exception('Não é possível visualizar informações anteriores à data ' . $inicioAno);
            }

        }  catch (\Exception $e) {
            throw $e;
        }

        $params = [
            'requestType' => 'Row',
            'row' => $row
        ];

        $response = $this->hcmServices->send($hcmServicesRoute, $params, $nrOrg);
        $reportLink = (string) $response['dataset'][$hcmServicesRoute]['link'];

        return $reportLink;
    }

    public function downloadReport(
        int $nrOrg,
        string $link
    ): array {
        $hcmServicesRoute = 'downloadReport';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'link' => $link
            ]
        ];

        $response = $this->hcmServices->send($hcmServicesRoute, $params, $nrOrg);
        $reportLink = (array) $response['dataset'][$hcmServicesRoute];

        return $reportLink;
    }

}
